==> ews-javascript-api-0.1.2.tgz: package/js/Core/ServiceObjects/ServiceObject.js <==
var Strings_1 = require("../../Strings");
var ServiceObjectPropertyException_1 = require("../../Exceptions/ServiceObjectPropertyException");
var PropertyBag_1 = require("../PropertyBag");
var XmlElementNames_1 = require("../../Core/XmlElementNames");
var PropertySet_1 = require("../PropertySet");
var EwsLogging_1 = require("../EwsLogging");
var ExtensionMethods_1 = require("../../ExtensionMethods");
var ServiceObject = (function () {
    function ServiceObject(service) {
        //EwsUtilities.ValidateParam(service, "service");
        //EwsUtilities.ValidateServiceObjectVersion(this, service.RequestedServerVersion);
        this.lockObject = {};
        this.Service = service;
        this.propertyBag = new PropertyBag_1.PropertyBag(this);
    }
    Object.defineProperty(ServiceObject.prototype, "PropertyBag", {
        get: function () { return this.propertyBag; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServiceObject.prototype, "Schema", {
        get: function () { return this.GetSchema(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServiceObject.prototype, "IsNew", {
        get: function () {
            return this.IsNewProxy();
            // var id = this.GetId();
            // debugger;
            // return id == null ? true : !id.IsValid;
        },
        enumerable: true,
        configurable: true
    });
    ServiceObject.prototype.IsNewProxy = function () {
        var id = this.GetId();
        debugger;
        return id == null ? true : !id.IsValid;
    };
    Object.defineProperty(ServiceObject.prototype, "IsDirty", {
        get: function () {
            return this.PropertyBag.IsDirty;
        },
        enumerable: true,
        configurable: true
    });
    /**This Indexer of c# */
    ServiceObject.prototype._getItem = function (propertyDefinition) {
        var propertyValue;
        var propDef = propertyDefinition;
        if (propDef != null) {
            debugger;
            return this.PropertyBag._getItem(propDef);
        }
        else {
            var extendedPropDef = propertyDefinition;
            if (extendedPropDef != null) {
                if (this.TryGetExtendedProperty(extendedPropDef, propertyValue)) {
                    return propertyValue;
                }
                else {
                    throw new ServiceObjectPropertyException_1.ServiceObjectPropertyException(Strings_1.Strings.MustLoadOrAssignPropertyBeforeAccess, propertyDefinition);
                }
            }
            else {
                // Other subclasses of PropertyDefinitionBase are not supported.
                throw new Error(ExtensionMethods_1.StringHelper.Format("not supported for property definition type: {0}", propertyDefinition.constructor));
            }
        }
    };
    ServiceObject.prototype.Changed = function () {
        if (this.OnChange != null) {
            this.OnChange(this);
        }
    };
    ServiceObject.prototype.ClearChangeLog = function () { this.PropertyBag.ClearChangeLog(); };
    ServiceObject.prototype.GetChangeXmlElementName = function () { return XmlElementNames_1.XmlElementNames.ItemChange; };
    ServiceObject.prototype.GetDeleteFieldXmlElementName = function () { return XmlElementNames_1.XmlElementNames.DeleteItemField; };
    ServiceObject.prototype.GetExtendedProperties = function () { return null; };
    ServiceObject.prototype.GetId = function () {
        var idPropertyDefinition = this.GetIdPropertyDefinition();
        var serviceId = { outValue: null };
        debugger;
        if (idPropertyDefinition != null) {
            this.PropertyBag.TryGetValue(idPropertyDefinition, serviceId);
        }
        return serviceId.outValue;
    };
    ServiceObject.prototype.GetIdPropertyDefinition = function () { return null; };
    ServiceObject.prototype.GetIsCustomDateTimeScopingRequired = function () { return false; };
    ServiceObject.prototype.GetIsTimeZoneHeaderRequired = function (isUpdateOperation) { return false; };
    ServiceObject.prototype.GetLoadedPropertyDefinitions = function () {
        var propDefs = [];
        for (var _i = 0, _a = this.PropertyBag.Properties.Keys; _i < _a.length; _i++) {
            var propDef = _a[_i];
            propDefs.push(propDef);
        }
        if (this.GetExtendedProperties() != null) {
            for (var _b = 0, _c = this.GetExtendedProperties().Items; _b < _c.length; _b++) {
                var extProp = _c[_b];
                propDefs.push(extProp.PropertyDefinition);
            }
        }
        return propDefs;
    };
    ServiceObject.prototype.GetMinimumRequiredServerVersion = function () { throw new Error("abstract method, must implement"); };
    ServiceObject.prototype.GetSchema = function () { throw new Error("abstract method, must implement"); };
    ServiceObject.prototype.GetSetFieldXmlElementName = function () { return XmlElementNames_1.XmlElementNames.SetItemField; };
    ServiceObject.prototype.GetXmlElementName = function () {
        debugger;
        throw new Error("ServiceObject.ts - getxmlelementname -  this must be overridden by derived class - can not use reflection to get class attribute in javascript");
        if (ExtensionMethods_1.StringHelper.IsNullOrEmpty(this.xmlElementName)) {
            this.xmlElementName = this.GetXmlElementNameOverride();
            EwsLogging_1.EwsLogging.Assert(!ExtensionMethods_1.StringHelper.IsNullOrEmpty(this.xmlElementName), "EwsObject.GetXmlElementName", ExtensionMethods_1.StringHelper.Format("The class {0} does not have an associated XML element name.", "unknown decendent of ServiceObject - in serviceObject.GetXmlElementname"));
        }
        return this.xmlElementName;
    };
    ServiceObject.prototype.GetXmlElementNameOverride = function () { return null; };
    ServiceObject.prototype.InternalDelete = function (deleteMode, sendCancellationsMode, affectedTaskOccurrences) { throw new Error("abstract method, must implement"); };
    ServiceObject.prototype.InternalLoad = function (propertySet) { throw new Error("abstract method, must implement"); };
    //Load(): any { throw new Error("ServiceObject.ts - Load : Not implemented."); }
    ServiceObject.prototype.Load = function (propertySet) {
        this.InternalLoad(propertySet || PropertySet_1.PropertySet.FirstClassProperties);
    };
    //LoadFromJson(jsonObject: JsonObject, service: ExchangeService, clearPropertyBag: boolean): any { throw new Error("ServiceObject.ts - LoadFromJson : Not implemented."); }
    //LoadFromJson(jsonServiceObject: JsonObject, service: ExchangeService, clearPropertyBag: boolean, requestedPropertySet: PropertySet, summaryPropertiesOnly: boolean): any { throw new Error("ServiceObject.ts - LoadFromJson : Not implemented."); }
    ServiceObject.prototype.LoadFromXmlJsObject = function (jsObject, service, clearPropertyBag, requestedPropertySet, summaryPropertiesOnly) {
        if (requestedPropertySet === void 0) { requestedPropertySet = null; }
        if (summaryPropertiesOnly === void 0) { summaryPropertiesOnly = false; }
        this.PropertyBag.LoadFromXmlJsObject(jsObject, service, clearPropertyBag, requestedPropertySet, summaryPropertiesOnly);
    };
    //LoadFromXml(reader: EwsServiceXmlReader, clearPropertyBag: boolean): any { throw new Error("could Not implemented.");
    //    this.PropertyBag.LoadFromXml(
    //        reader,
    //        clearPropertyBag,
    //        null,       /* propertySet */
    //        false);     /* summaryPropertiesOnly */
    //}
    /// <summary>
    /// Throws exception if this is a new service object.
    /// </summary>
    ServiceObject.prototype.ThrowIfThisIsNew = function () {
        if (this.IsNew) {
            throw new Error("service object does not have id"); //InvalidOperationException(Strings.ServiceObjectDoesNotHaveId);
        }
    };
    /// <summary>
    /// Throws exception if this is not a new service object.
    /// </summary>
    ServiceObject.prototype.ThrowIfThisIsNotNew = function () {
        if (!this.IsNew) {
            throw new Error("service object already have id"); //InvalidOperationException(Strings.ServiceObjectAlreadyHasId);
        }
    };
    //ToJson(service: ExchangeService, isUpdateOperation: boolean): any { return this.PropertyBag.ToJson(service, isUpdateOperation);}
    ServiceObject.prototype.TryGetExtendedProperty = function (propertyDefinition, propertyValue) {
        var propertyCollection = this.GetExtendedProperties();
        if ((propertyCollection != null) &&
            propertyCollection.TryGetValue(propertyDefinition, propertyValue)) {
            return true;
        }
        else {
            propertyValue.outValue = null; //default(T);
            return false;
        }
    };
    //TryGetProperty<T>(propertyDefinition: PropertyDefinitionBase, propertyValue: any): boolean { throw new Error("Need implementation."); }
    //TryGetProperty(propertyDefinition: PropertyDefinitionBase, propertyValue: any): boolean { throw new Error("ServiceObject.ts - TryGetProperty : Not implemented."); }
    ServiceObject.prototype.TryGetProperty = function (propertyDefinition, propertyValue) {
        var propDef = propertyDefinition; // as PropertyDefinition;
        debugger; //todo: fix for compatibility checking, if this is propertydefinition or propertydefinitionbase
        if (propDef != null) {
            return this.PropertyBag.TryGetPropertyAs(propDef, propertyValue);
        }
        else {
            debugger; //todo: check for compatibility of extendedpropertydefition or propertydefition.
            var extPropDef = propertyDefinition; // as ExtendedPropertyDefinition;
            if (extPropDef != null) {
                return this.TryGetExtendedProperty(extPropDef, propertyValue);
            }
            else {
                // Other subclasses of PropertyDefinitionBase are not supported.
                throw new Error(ExtensionMethods_1.StringHelper.Format(Strings_1.Strings.OperationNotSupportedForPropertyDefinitionType, propertyDefinition.Type)); //NotSupportedException
            }
        }
    };
    ServiceObject.prototype.Validate = function () { this.PropertyBag.Validate(); };
    //WriteToJsonForUpdate(service: ExchangeService): any { throw new Error("ServiceObject.ts - WriteToJsonForUpdate : Not implemented."); }
    ServiceObject.prototype.WriteToXml = function (writer) { this.PropertyBag.WriteToXml(writer); };
    ServiceObject.prototype.WriteToXmlForUpdate = function (writer) { this.PropertyBag.WriteToXmlForUpdate(writer); };
    //created this to keep item and folder object away frmo here. modularization would fail and create a larger file
    ServiceObject.prototype.IsFolderInstance = function () { return false; }; //only folder instance would return true.
    ServiceObject.prototype.IsItemInstance = function () { return false; }; //only item instance would return true.
    Object.defineProperty(ServiceObject.prototype, "IsAttachment", {
        get: function () { return false; } //only item instance would return true.
        ,
        enumerable: true,
        configurable: true
    });
    return ServiceObject;
})();
exports.ServiceObject = ServiceObject;

==> ews-javascript-api-0.1.6.tgz: package/js/Core/ServiceObjects/ServiceObject.js <==
var Strings_1 = require("../../Strings");
var ServiceObjectPropertyException_1 = require("../../Exceptions/ServiceObjectPropertyException");
var PropertyBag_1 = require("../PropertyBag");
var XmlElementNames_1 = require("../../Core/XmlElementNames");
var PropertySet_1 = require("../PropertySet");
var EwsLogging_1 = require("../EwsLogging");
var ExtensionMethods_1 = require("../../ExtensionMethods");
var ServiceObject = (function () {
    function ServiceObject(service) {
        //EwsUtilities.ValidateParam(service, "service");
        //EwsUtilities.ValidateServiceObjectVersion(this, service.RequestedServerVersion);
        this.lockObject = {};
        this.OnChange = [];
        this.Service = service;
        this.propertyBag = new PropertyBag_1.PropertyBag(this);
    }
    Object.defineProperty(ServiceObject.prototype, "PropertyBag", {
        get: function () { return this.propertyBag; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServiceObject.prototype, "Schema", {
        get: function () { return this.GetSchema(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServiceObject.prototype, "IsNew", {
        get: function () {
            var id = this.GetId();
            return id == null ? true : !id.IsValid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServiceObject.prototype, "IsDirty", {
        get: function () {
            return this.PropertyBag.IsDirty;
        },
        enumerable: true,
        configurable: true
    });
    /**This Indexer of c# */
    ServiceObject.prototype._getItem = function (propertyDefinition) {
        var propertyValue;
        var propDef = propertyDefinition;
        if (propDef != null) {
            //todo: check for propertydefinitionbase type or child type;
            return this.PropertyBag._getItem(propDef);
        }
        else {
            var extendedPropDef = propertyDefinition;
            if (extendedPropDef != null) {
                if (this.TryGetExtendedProperty(extendedPropDef, propertyValue)) {
                    return propertyValue;
                }
                else {
                    throw new ServiceObjectPropertyException_1.ServiceObjectPropertyException(Strings_1.Strings.MustLoadOrAssignPropertyBeforeAccess, propertyDefinition);
                }
            }
            else {
                // Other subclasses of PropertyDefinitionBase are not supported.
                throw new Error(ExtensionMethods_1.StringHelper.Format("not supported for property definition type: {0}", propertyDefinition.constructor));
            }
        }
    };
    ServiceObject.prototype.Changed = function () {
        if (this.OnChange != null) {
            for (var _i = 0, _a = this.OnChange; _i < _a.length; _i++) {
                var changeDelegate = _a[_i];
                changeDelegate(this);
            }
        }
    };
    ServiceObject.prototype.ClearChangeLog = function () { this.PropertyBag.ClearChangeLog(); };
    ServiceObject.prototype.GetChangeXmlElementName = function () { return XmlElementNames_1.XmlElementNames.ItemChange; };
    ServiceObject.prototype.GetDeleteFieldXmlElementName = function () { return XmlElementNames_1.XmlElementNames.DeleteItemField; };
    ServiceObject.prototype.GetExtendedProperties = function () { return null; };
    ServiceObject.prototype.GetId = function () {
        var idPropertyDefinition = this.GetIdPropertyDefinition();
        var serviceId = { outValue: null };
        if (idPropertyDefinition != null) {
            this.PropertyBag.TryGetValue(idPropertyDefinition, serviceId);
        }
        return serviceId.outValue;
    };
    ServiceObject.prototype.GetIdPropertyDefinition = function () { return null; };
    ServiceObject.prototype.GetIsCustomDateTimeScopingRequired = function () { return false; };
    ServiceObject.prototype.GetIsTimeZoneHeaderRequired = function (isUpdateOperation) { return false; };
    ServiceObject.prototype.GetLoadedPropertyDefinitions = function () {
        var propDefs = [];
        for (var _i = 0, _a = this.PropertyBag.Properties.Keys; _i < _a.length; _i++) {
            var propDef = _a[_i];
            propDefs.push(propDef);
        }
        if (this.GetExtendedProperties() != null) {
            for (var _b = 0, _c = this.GetExtendedProperties().Items; _b < _c.length; _b++) {
                var extProp = _c[_b];
                propDefs.push(extProp.PropertyDefinition);
            }
        }
        return propDefs;
    };
    ServiceObject.prototype.GetMinimumRequiredServerVersion = function () { throw new Error("abstract method, must implement"); };
    ServiceObject.prototype.GetSchema = function () { throw new Error("abstract method, must implement"); };
    ServiceObject.prototype.GetSetFieldXmlElementName = function () { return XmlElementNames_1.XmlElementNames.SetItemField; };
    ServiceObject.prototype.GetXmlElementName = function () {
        throw new Error("ServiceObject.ts - GetXmlElementName -  this must be overridden by derived class - can not use reflection to get class attribute in javascript");
        if (ExtensionMethods_1.StringHelper.IsNullOrEmpty(this.xmlElementName)) {
            this.xmlElementName = this.GetXmlElementNameOverride();
            EwsLogging_1.EwsLogging.Assert(!ExtensionMethods_1.StringHelper.IsNullOrEmpty(this.xmlElementName), "EwsObject.GetXmlElementName", ExtensionMethods_1.StringHelper.Format("The class {0} does not have an associated XML element name.", "unknown decendent of ServiceObject - in serviceObject.GetXmlElementname"));
        }
        return this.xmlElementName;
    };
    ServiceObject.prototype.GetXmlElementNameOverride = function () { return null; };
    ServiceObject.prototype.InternalDelete = function (deleteMode, sendCancellationsMode, affectedTaskOccurrences) { throw new Error("abstract method, must implement"); };
    ServiceObject.prototype.InternalLoad = function (propertySet) { throw new Error("abstract method, must implement"); };
    //Load(): any { throw new Error("ServiceObject.ts - Load : Not implemented."); }
    ServiceObject.prototype.Load = function (propertySet) {
        return this.InternalLoad(propertySet || PropertySet_1.PropertySet.FirstClassProperties);
    };
    //LoadFromJson(jsonObject: JsonObject, service: ExchangeService, clearPropertyBag: boolean): any { throw new Error("ServiceObject.ts - LoadFromJson : Not implemented."); }
    //LoadFromJson(jsonServiceObject: JsonObject, service: ExchangeService, clearPropertyBag: boolean, requestedPropertySet: PropertySet, summaryPropertiesOnly: boolean): any { throw new Error("ServiceObject.ts - LoadFromJson : Not implemented."); }
    ServiceObject.prototype.LoadFromXmlJsObject = function (jsObject, service, clearPropertyBag, requestedPropertySet, summaryPropertiesOnly) {
        if (requestedPropertySet === void 0) { requestedPropertySet = null; }
        if (summaryPropertiesOnly === void 0) { summaryPropertiesOnly = false; }
        this.PropertyBag.LoadFromXmlJsObject(jsObject, service, clearPropertyBag, requestedPropertySet, summaryPropertiesOnly);
    };
    //LoadFromXml(reader: EwsServiceXmlReader, clearPropertyBag: boolean): any { throw new Error("could Not implemented.");
    //    this.PropertyBag.LoadFromXml(
    //        reader,
    //        clearPropertyBag,
    //        null,       /* propertySet */
    //        false);     /* summaryPropertiesOnly */
    //}
    /// <summary>
    /// Throws exception if this is a new service object.
    /// </summary>
    ServiceObject.prototype.ThrowIfThisIsNew = function () {
        if (this.IsNew) {
            throw new Error("service object does not have id"); //InvalidOperationException(Strings.ServiceObjectDoesNotHaveId);
        }
    };
    /// <summary>
    /// Throws exception if this is not a new service object.
    /// </summary>
    ServiceObject.prototype.ThrowIfThisIsNotNew = function () {
        if (!this.IsNew) {
            throw new Error("service object already have id"); //InvalidOperationException(Strings.ServiceObjectAlreadyHasId);
        }
    };
    //ToJson(service: ExchangeService, isUpdateOperation: boolean): any { return this.PropertyBag.ToJson(service, isUpdateOperation);}
    ServiceObject.prototype.TryGetExtendedProperty = function (propertyDefinition, propertyValue) {
        var propertyCollection = this.GetExtendedProperties();
        if ((propertyCollection != null) &&
            propertyCollection.TryGetValue(propertyDefinition, propertyValue)) {
            return true;
        }
        else {
            propertyValue.outValue = null; //default(T);
            return false;
        }
    };
    //TryGetProperty<T>(propertyDefinition: PropertyDefinitionBase, propertyValue: any): boolean { throw new Error("Need implementation."); }
    //TryGetProperty(propertyDefinition: PropertyDefinitionBase, propertyValue: any): boolean { throw new Error("ServiceObject.ts - TryGetProperty : Not implemented."); }
    ServiceObject.prototype.TryGetProperty = function (propertyDefinition, propertyValue) {
        var propDef = propertyDefinition; // as PropertyDefinition;
        debugger; //todo: fix for compatibility checking, if this is propertydefinition or propertydefinitionbase
        if (propDef != null) {
            return this.PropertyBag.TryGetPropertyAs(propDef, propertyValue);
        }
        else {
            debugger; //todo: check for compatibility of extendedpropertydefition or propertydefition.
            var extPropDef = propertyDefinition; // as ExtendedPropertyDefinition;
            if (extPropDef != null) {
                return this.TryGetExtendedProperty(extPropDef, propertyValue);
            }
            else {
                // Other subclasses of PropertyDefinitionBase are not supported.
                throw new Error(ExtensionMethods_1.StringHelper.Format(Strings_1.Strings.OperationNotSupportedForPropertyDefinitionType, propertyDefinition.Type)); //NotSupportedException
            }
        }
    };
    ServiceObject.prototype.Validate = function () { this.PropertyBag.Validate(); };
    //WriteToJsonForUpdate(service: ExchangeService): any { throw new Error("ServiceObject.ts - WriteToJsonForUpdate : Not implemented."); }
    ServiceObject.prototype.WriteToXml = function (writer) { this.PropertyBag.WriteToXml(writer); };
    ServiceObject.prototype.WriteToXmlForUpdate = function (writer) { this.PropertyBag.WriteToXmlForUpdate(writer); };
    Object.defineProperty(ServiceObject.prototype, "InstanceType", {
        //created this to help find serviceobject type, ServiceObjectInstance instanceof Item/Folder/Attachment fails by creating circular dependency in javascript/typescript
        get: function () { return "ServiceObject"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServiceObject.prototype, "IsAttachment", {
        //created this to keep item and folder object away frmo here. modularization would fail and create a larger file
        get: function () { return false; } //only item instance would return true.
        ,
        enumerable: true,
        configurable: true
    });
    return ServiceObject;
})();
exports.ServiceObject = ServiceObject;
